package de.jottyfan.timetrack.spring.done;

import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

/**
 * 
 * @author henkej
 *
 */
@Controller
public class DoneController {
	private static final Logger LOGGER = LogManager.getLogger(DoneController.class);

	private final HttpServletRequest request;

	@Autowired
	private IDoneService doneService;

	@Autowired
	public DoneController(HttpServletRequest request) {
		this.request = request;
	}

	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/done/list")
	public String getList(@ModelAttribute DoneModel doneModel, Model model) {
		String username = doneService.getCurrentUser(request);
		List<DoneBean> list = doneService.getList(doneModel.getDay(), username);
		model.addAttribute("doneList", list);
		model.addAttribute("doneModel", doneModel);
		model.addAttribute("projectList", doneService.getProjects());
		model.addAttribute("moduleList", doneService.getModules());
		model.addAttribute("jobList", doneService.getJobs());
		model.addAttribute("billingList", doneService.getBillings());
		return "done/list";
	}

	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/done/add", method = RequestMethod.GET)
	public String toAdd(Model model) {
		return toItem(null, model);
	}

	@RolesAllowed("timetrack_user")
  @GetMapping("/done/edit/{id}")
	public String toItem(@PathVariable Integer id, Model model) {
		DoneBean bean = doneService.getBean(id);
		if (bean == null) {
			bean = new DoneBean(); // the add case
		}
		model.addAttribute("doneBean", bean);
		model.addAttribute("projectList", doneService.getProjects());
		model.addAttribute("moduleList", doneService.getModules());
		model.addAttribute("jobList", doneService.getJobs());
		model.addAttribute("billingList", doneService.getBillings());
		return "done/item";
	}
	
	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/done/upsert", method = RequestMethod.POST)
	public String doUpsert(Model model, @ModelAttribute DoneBean bean) {
		String username = doneService.getCurrentUser(request);
		Integer amount = doneService.doUpsert(bean, username);
		DoneModel doneModel = new DoneModel();
		doneModel.setDay(bean.getLocalDate());
		return amount.equals(1) ? getList(doneModel, model) : toItem(bean.getPk(), model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/delete/{id}")
	public String doDelete(@PathVariable Integer id, Model model) {
		DoneBean bean = doneService.getBean(id);
		Integer amount = doneService.doDelete(id);
		DoneModel doneModel = new DoneModel();
		doneModel.setDay(bean.getLocalDate());
		return amount.equals(1) ? getList(doneModel, model) : toItem(id, model);
	}
}
