package de.jottyfan.timetrack.spring.done.impl;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.keycloak.KeycloakSecurityContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import de.jottyfan.timetrack.db.done.tables.records.TBillingRecord;
import de.jottyfan.timetrack.db.done.tables.records.TJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.TModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.spring.done.DoneBean;
import de.jottyfan.timetrack.spring.done.IDoneService;
import de.jottyfan.timetrack.spring.note.impl.NoteService;

/**
 * 
 * @author henkej
 *
 */
@Service
@Transactional(transactionManager = "transactionManager")
public class DoneService implements IDoneService {
	private static final Logger LOGGER = LogManager.getLogger(NoteService.class);

	@Autowired
	private DSLContext dsl;

	@Override
	public String getCurrentUser(HttpServletRequest request) {
		KeycloakSecurityContext ksc = (KeycloakSecurityContext) request
				.getAttribute(KeycloakSecurityContext.class.getName());
		return ksc == null ? "" : ksc.getIdToken().getPreferredUsername();
	}

	@Override
	public List<DoneBean> getList(LocalDate day, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			if (userId == null) {
				LOGGER.warn("userId of user {} is null", username);
			}
			return gw.getAllOfDay(day, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	@Override
	public DoneBean getBean(Integer id) {
		try {
			return new DoneGateway(dsl).getBean(id);
		} catch (Exception e) {
			LOGGER.error(e);
			return null;
		}
	}

	@Override
	public List<TProjectRecord> getProjects() {
		try {
			return new DoneGateway(dsl).getAllProjects();
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	@Override
	public List<TModuleRecord> getModules() {
		try {
			return new DoneGateway(dsl).getAllModules();
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	@Override
	public List<TJobRecord> getJobs() {
		try {
			return new DoneGateway(dsl).getAllJobs();
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	@Override
	public List<TBillingRecord> getBillings() {
		try {
			return new DoneGateway(dsl).getAllBillings();
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	@Override
	public Integer doUpsert(DoneBean bean, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			return gw.upsert(bean, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

	@Override
	public Integer doDelete(Integer id) {
		try {
			return new DoneGateway(dsl).delete(id);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

}
